print("import: 'scipy'")
import scipy

print("import: 'scipy.cluster'")
import scipy.cluster

print("import: 'scipy.cluster.hierarchy'")
import scipy.cluster.hierarchy

print("import: 'scipy.cluster.vq'")
import scipy.cluster.vq

print("import: 'scipy.constants'")
import scipy.constants

print("import: 'scipy.fft'")
import scipy.fft

print("import: 'scipy.fftpack'")
import scipy.fftpack

print("import: 'scipy.integrate'")
import scipy.integrate

print("import: 'scipy.interpolate'")
import scipy.interpolate

print("import: 'scipy.io'")
import scipy.io

print("import: 'scipy.io.arff'")
import scipy.io.arff

print("import: 'scipy.io.matlab'")
import scipy.io.matlab

print("import: 'scipy.io.wavfile'")
import scipy.io.wavfile

print("import: 'scipy.linalg'")
import scipy.linalg

print("import: 'scipy.linalg.blas'")
import scipy.linalg.blas

print("import: 'scipy.linalg.cython_blas'")
import scipy.linalg.cython_blas

print("import: 'scipy.linalg.cython_lapack'")
import scipy.linalg.cython_lapack

print("import: 'scipy.linalg.interpolative'")
import scipy.linalg.interpolative

print("import: 'scipy.linalg.lapack'")
import scipy.linalg.lapack

print("import: 'scipy.misc'")
import scipy.misc

print("import: 'scipy.ndimage'")
import scipy.ndimage

print("import: 'scipy.odr'")
import scipy.odr

print("import: 'scipy.optimize'")
import scipy.optimize

print("import: 'scipy.signal'")
import scipy.signal

print("import: 'scipy.signal.windows'")
import scipy.signal.windows

print("import: 'scipy.sparse'")
import scipy.sparse

print("import: 'scipy.sparse.csgraph'")
import scipy.sparse.csgraph

print("import: 'scipy.sparse.linalg'")
import scipy.sparse.linalg

print("import: 'scipy.spatial'")
import scipy.spatial

print("import: 'scipy.spatial.distance'")
import scipy.spatial.distance

print("import: 'scipy.spatial.transform'")
import scipy.spatial.transform

print("import: 'scipy.special'")
import scipy.special

print("import: 'scipy.stats'")
import scipy.stats

print("import: 'scipy.stats.contingency'")
import scipy.stats.contingency

print("import: 'scipy.stats.distributions'")
import scipy.stats.distributions

print("import: 'scipy.stats.mstats'")
import scipy.stats.mstats

print("import: 'scipy.stats.qmc'")
import scipy.stats.qmc

print("import: 'scipy.stats.sampling'")
import scipy.stats.sampling

